/*
 * Worldvisions Weaver Software:
 *   Copyright (C) 1997-2002 Net Integration Technologies, Inc.
 *
 * Standalone main program for Fast Forward.
 *
 */

#include "wvfastfwd.h"
#include "wvsyslog.h"
#include <time.h>
#include <signal.h>
#include <unistd.h>

#define INIFILE "/etc/fastforward.conf"

static volatile bool want_to_die    = false;
static volatile bool want_to_reload = true;


void signal_handler( int signum )
/*******************************/
{
    fprintf( stderr, "Caught signal %d; cleaning up and terminating.\n",
	     signum );
    want_to_die = true;
}

void reload_handler( int )
/************************/
{
    want_to_reload = true;
}

void usage( char * argv0 )
/************************/
{
    fprintf( stderr, "Usage: %s [-d] [-c configfile]\n"
	     "   -d  enable debug mode (log to stderr, not syslog)\n"
	     "   -c  use a different config file (default is " INIFILE "\n"
	     "   -w  change IP firewall (iptables) automatically\n",
	     argv0 );
}

int main( int argc, char * argv[] )
/*********************************/
{
    char * inifile    = INIFILE;
    bool   use_syslog = true;
    int    opt;
    
    while( ( opt = getopt( argc, argv, "dc:w?" ) ) != EOF ) {
	switch( opt ) {
	case 'd':
	    use_syslog = false;
	    break;
	case 'c':
	    inifile = optarg;
	    break;
	case 'w':
	    WvIPFirewall::enable = true;
	    break;
	case '?':
	    usage( argv[0] );
	    return( 1 );
	}
    }

    WvConf cfg( inifile );
    if( !cfg.isok() || !cfg.isclean() ) {
    	fprintf( stderr, "Fast Forward: unable to open %s\n", INIFILE );
    	return( -1 );
    }
    
    signal( SIGINT, signal_handler );
    signal( SIGTERM, signal_handler );
    signal( SIGHUP, reload_handler );
    signal( SIGPIPE, SIG_IGN );
    
    WvLogRcv * syslog;
    
    if( use_syslog )
	syslog = new WvSyslog( "fastfwd", false );
    else
	syslog = NULL;
    
    WvFastForward fwd( cfg );
    want_to_reload = true;

    while( !want_to_die ) {
	assert(fwd.isok());
	
	if( want_to_reload ) {
	    fwd.reload_config();
	    want_to_reload = false;
	}
	
	if( fwd.select( 10000 ) )
	    fwd.callback();
	fwd.clean_up();
    }
    
    if( syslog )
	delete syslog;

    return( 0 );
}
